@@ -24,6 +24,7 @@
 #include <X11/Xmd.h>
 #include <X11/Xauth.h>
 #include <netinet/in.h>
+#include <time.h>
 
 #define STX 0x2			/* Start of txt */
 #define BEL 0x7			/* Bell, used to interrupt login for
@@ -34,16 +35,15 @@
 #define TYPE_FLEXI 3		/* Local Flexi X server */
 #define TYPE_FLEXI_XNEST 4	/* Local Flexi Xnest server */
 
-#define SERVER_IS_LOCAL(d) (d->type == TYPE_LOCAL || \
-			    d->type == TYPE_FLEXI || \
-			    d->type == TYPE_FLEXI_XNEST)
-#define SERVER_IS_FLEXI(d) (d->type == TYPE_FLEXI || \
-			    d->type == TYPE_FLEXI_XNEST)
+#define SERVER_IS_LOCAL(d) ((d)->type == TYPE_LOCAL || \
+			    (d)->type == TYPE_FLEXI || \
+			    (d)->type == TYPE_FLEXI_XNEST)
+#define SERVER_IS_FLEXI(d) ((d)->type == TYPE_FLEXI || \
+			    (d)->type == TYPE_FLEXI_XNEST)
 
-#define SERVER_SUCCESS 0	/* X server default */
-#define SERVER_FAILURE 1	/* X server default */
+/* These are the servstat values, also used as server
+ * process exit codes */
 #define SERVER_TIMEOUT 2	/* Server didn't start */
-#define SERVER_NOTFOUND 127	/* X server default */
 #define SERVER_DEAD 250		/* Server stopped */
 #define SERVER_STARTED 251	/* Server started but not ready for connections yet */
 #define SERVER_RUNNING 252	/* Server running and ready for connections */
@@ -52,7 +52,7 @@
 /* DO NOTE USE 1, that's used as error if x connection fails usually */
 /* Note that there is no reasons why these were a power of two, and note
  * that they have to fit in 256 */
-#define DISPLAY_SUCCESS 0	/* All systems are go */
+/* These are the exit codes */
 #define DISPLAY_REMANAGE 2	/* Restart display */
 #define DISPLAY_ABORT 4		/* Houston, we have a problem */
 #define DISPLAY_REBOOT 8	/* Rebewt */
@@ -60,14 +60,16 @@
 #define DISPLAY_SUSPEND 17	/* Suspend */
 #define DISPLAY_CHOSEN 20	/* successful chooser session,
 				   restart display */
-#define DISPLAY_DEAD 32		/* Display not configured/started yet */
 #define DISPLAY_XFAILED 64	/* X failed */
 #define DISPLAY_RESTARTGDM 128	/* Restart GDM */
 
-#define XDMCP_DEAD 0
-#define XDMCP_PENDING 1
-#define XDMCP_MANAGED 2
-
+enum {
+	DISPLAY_UNBORN /* Not yet started */,
+	DISPLAY_ALIVE /* Yay! we're alive (non-xdmcp) */,
+	XDMCP_PENDING /* Pending XDMCP display */,
+	XDMCP_MANAGED /* Managed XDMCP display */,
+	DISPLAY_DEAD /* Left for dead */
+};
 
 /* Opcodes for the highly sophisticated protocol used for
  * daemon<->greeter communications */
@@ -76,7 +78,7 @@
  * protocol changes */
 #define GDM_GREETER_PROTOCOL_VERSION "1"
 
-#define GDM_MSGERR     'D'
+#define GDM_MSG        'D'
 #define GDM_NOECHO     'U'
 #define GDM_PROMPT     'N'
 #define GDM_SESS       'G'
@@ -100,7 +102,8 @@
 #define GDM_NEEDPIC    '#' /* need a user picture?, sent after greeter
 			    *  is started */
 #define GDM_READPIC    '%' /* Send a user picture in a temp file */
-#define GDM_ERRBOX     'e' /* Puts string in a popup error box */
+#define GDM_ERRBOX     'e' /* Puts string in the error box */
+#define GDM_ERRDLG     'E' /* Puts string up in an error dialog */
 
 /* Different login interruptions */
 #define GDM_INTERRUPT_TIMED_LOGIN 'T'
@@ -118,7 +121,8 @@
 #define GDM_KEY_AUTOMATICLOGIN_ENABLE "daemon/AutomaticLoginEnable=true"
 #define GDM_KEY_AUTOMATICLOGIN "daemon/AutomaticLogin="
 #define GDM_KEY_ALWAYSRESTARTSERVER "daemon/AlwaysRestartServer=false"
-#define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_BINDIR "/gdmlogin --disable-sound --disable-crash-dialog"
+#define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_BINDIR "/gdmlogin"
+#define GDM_KEY_REMOTEGREETER "daemon/RemoteGreeter=" EXPANDED_BINDIR "/gdmlogin"
 #define GDM_KEY_GROUP "daemon/Group=gdm"
 #define GDM_KEY_HALT "daemon/HaltCommand=/sbin/shutdown -h now"
 #define GDM_KEY_INITDIR "daemon/DisplayInitDir=" EXPANDED_SYSCONFDIR "/gdm/Init"
@@ -130,7 +134,6 @@
 #define GDM_KEY_PRESESS "daemon/PreSessionScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PreSession/"
 #define GDM_KEY_FAILSAFE_XSERVER "daemon/FailsafeXServer="
 #define GDM_KEY_XKEEPSCRASHING "daemon/XKeepsCrashing=" EXPANDED_SYSCONFDIR "/gdm/XKeepsCrashing"
-#define GDM_KEY_XKEEPSCRASHING_CONFIGURATORS "daemon/XKeepsCrashingConfigurators=/usr/bin/X11/XF86Setup /usr/bin/X11/Xconfigurator"
 #define GDM_KEY_REBOOT "daemon/RebootCommand=/sbin/shutdown -r now"
 #define GDM_KEY_ROOTPATH "daemon/RootPath=/sbin:/usr/sbin:/bin:/usr/bin:/usr/bin/X11:/usr/local/bin:" EXPANDED_BINDIR
 #define GDM_KEY_GNOMEDEFAULTSESSION "daemon/GnomeDefaultSession=" EXPANDED_DATADIR "/gnome/default.session"
@@ -169,7 +172,6 @@
 #define GDM_KEY_SESSIONMAXFILE "security/SessionMaxFile=524288"
 #define GDM_KEY_RELAXPERM "security/RelaxPermissions=0"
 #define GDM_KEY_RETRYDELAY "security/RetryDelay=3"
-#define GDM_KEY_VERBAUTH "security/VerboseAuth=true"
 
 #define GDM_KEY_XDMCP "xdmcp/Enable=false"
 #define GDM_KEY_MAXPEND "xdmcp/MaxPending=4"
@@ -191,7 +193,6 @@
 #define GDM_KEY_EXCLUDE "greeter/Exclude=bin,daemon,adm,lp,sync,shutdown,halt,mail,news,uucp,operator,nobody,gdm,postgres,pvm"
 #define GDM_KEY_FACE "greeter/DefaultFace=" EXPANDED_PIXMAPDIR "nobody.png"
 #define GDM_KEY_FACEDIR "greeter/GlobalFaceDir=" EXPANDED_DATADIR "/faces/"
-#define GDM_KEY_FONT "greeter/Font=-adobe-helvetica-bold-r-normal-*-*-180-*-*-*-*-*-*"
 #define GDM_KEY_ICON "greeter/Icon=" EXPANDED_PIXMAPDIR "/gdm.xpm"
 #define GDM_KEY_LOCALE "greeter/DefaultLocale=english"
 #define GDM_KEY_LOCFILE "greeter/LocaleFile=" EXPANDED_LOCALEDIR "/locale.alias"
@@ -270,6 +271,7 @@ struct _GdmDisplay {
     guint8 dispstat;
     guint16 dispnum;
     guint8 servstat;
+    time_t starttime;
     guint8 type;
     pid_t greetpid;
     pid_t servpid;
@@ -311,6 +313,10 @@ struct _GdmDisplay {
     char *xnest_auth_file;
     uid_t server_uid;
     GdmConnection *socket_conn;
+
+    /* Notification connection */
+    int master_notify_fd;  /* write part of the connection */
+    GdmConnection *slave_notify_conn; /* read part of the connection */
 };
 
 typedef struct _GdmXServer GdmXServer;
@@ -370,25 +376,22 @@ enum {
 	GDM_XDMCP_LAST_OPCODE /*just a marker, not an opcode */
 };
 
-typedef gboolean (*GSignalFunc) (gint8		signal,
+typedef gboolean (*GDMSignalFunc) (gint8	signal,
 				 gpointer	data);
-guint	g_signal_add		(gint8		signal,
-				 GSignalFunc	function,
+guint	gdm_signal_add		(gint8		signal,
+				 GDMSignalFunc	function,
 				 gpointer    	data);
-guint   g_signal_add_full       (gint           priority,
+guint   gdm_signal_add_full     (gint           priority,
 				 gint8          signal,
-				 GSignalFunc    function,
+				 GDMSignalFunc  function,
 				 gpointer       data,
 				 GDestroyNotify destroy);
-void    g_signal_notify         (gint8          signal);
+void    gdm_signal_notify       (gint8          signal);
 
 /* If id == NULL, then get the first X server */
 GdmXServer *	gdm_find_x_server	(const char *id);
 
 
-void gdm_run (void);
-void gdm_quit (void);
-
 /* primitive protocol for controlling the daemon from slave
  * or gdmconfig or whatnot */
 
@@ -412,6 +415,16 @@ void gdm_quit (void);
 	/* 5 = Xnest can't connect */
 #define GDM_SOP_FLEXI_OK     "FLEXI_OK" /* <slave pid> */
 #define GDM_SOP_SOFT_RESTART "SOFT_RESTART" /* no arguments */
+#define GDM_SOP_START_NEXT_LOCAL "START_NEXT_LOCAL" /* no arguments */
+#define GDM_SOP_HUP_ALL_GREETERS "HUP_ALL_GREETERS" /* no arguments */
+
+/* Notification protocol */
+#define GDM_NOTIFY_ALLOWREMOTEROOT "AllowRemoteRoot" /* <true/false as int> */
+#define GDM_NOTIFY_ALLOWROOT "AllowRoot" /* <true/false as int> */
+#define GDM_NOTIFY_ALLOWREMOTEAUTOLOGIN "AllowRemoteAutoLogin" /* <true/false as int> */
+#define GDM_NOTIFY_RETRYDELAY "RetryDelay" /* <seconds> */
+#define GDM_NOTIFY_GREETER "Greeter" /* <greeter binary> */
+#define GDM_NOTIFY_REMOTEGREETER "RemoteGreeter" /* <greeter binary> */
 
 #define GDM_SUP_SOCKET "/tmp/.gdm_socket"
 
@@ -511,6 +524,47 @@ void gdm_quit (void);
  *   (that is, it is an xnest inside another console display) it is listed
  *   and instead of vt, it lists the parent display in standard form.
  */
+#define GDM_SUP_UPDATE_CONFIG "UPDATE_CONFIG" /* <key> */
+/* AUTH_LOCAL: Tell the daemon to update config of some key.  Any user
+ *             can really request that values are re-read but the daemon
+ *             caches the last date of the config file so a user can't
+ *             actually change any values unless they can write the
+ *             config file.  The keys that are currently supported are:
+ *   		 security/AllowRoot (2.3.90.2)
+ *   		 security/AllowRemoteRoot (2.3.90.2)
+ *   		 security/AllowRemoteAutoLogin (2.3.90.2)
+ *   		 security/RetryDelay (2.3.90.2)
+ *   		 daemon/Greeter (2.3.90.2)
+ *   		 daemon/RemoteGreeter (2.3.90.2)
+ *   		 xdmcp/Enable (2.3.90.2)
+ *   		 xdmcp/Port (2.3.90.2)
+ *   		 xdmcp/PARAMETERS (2.3.90.2) (pseudokey, all the parameters)
+ *			xdmcp/MaxPending
+ *			xdmcp/MaxSessions
+ *			xdmcp/MaxWait
+ *			xdmcp/DisplaysPerHost
+ *			xdmcp/HonorIndirect
+ *			xdmcp/MaxPendingIndirect
+ *			xdmcp/MaxWaitIndirect
+ *			xdmcp/PingInterval (only affects new connections)
+ * Supported since: 2.3.90.2
+ * Arguments:  <key>
+ *   <key> is just the base part of the key such as "security/AllowRemoteRoot"
+ * Answers:
+ *   OK
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      50 = Unsupported key
+ *      999 = Unknown error
+ */
+#define GDM_SUP_GREETERPIDS  "GREETERPIDS" /* None */
+/* GREETERPIDS: List all greeter pids so that one can send HUP to them
+ * for config rereading.  Of course one must be root to do that.
+ * Supported since: 2.3.90.2
+ * Arguments:  None
+ * Answers:
+ *   OK <pid>;<pid>;...
+ */
 #define GDM_SUP_CLOSE        "CLOSE" /* no arguments */
 /* CLOSE Answers: None
  * Supported since: 2.2.4.0
