@@ -19,8 +19,9 @@
 /* This file contains functions for controlling local X servers */
 
 #include <config.h>
-#include <gnome.h>
+#include <libgnome/libgnome.h>
 #include <stdio.h>
+#include <unistd.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include <pwd.h>
@@ -31,6 +32,7 @@
 #include <strings.h>
 #include <signal.h>
 #include <errno.h>
+#include <time.h>
 #include <X11/Xlib.h>
 
 #include <vicious.h>
@@ -43,8 +45,6 @@
 #include "auth.h"
 #include "slave.h"
 
-static const gchar RCSid[]="$Id: server.c,v 1.39 2001/10/09 04:05:18 jirka Exp $";
-
 #define SERVER_WAIT_ALARM 10
 
 
@@ -60,12 +60,14 @@ extern gchar *GdmServAuthDir;
 extern gchar *GdmLogDir;
 extern gchar *GdmStandardXServer;
 extern gboolean GdmXdmcp;
-extern sigset_t sysmask;
 extern gint high_display_num;
 extern pid_t extra_process;
+extern int extra_status;
 
 /* Global vars */
 static GdmDisplay *d = NULL;
+static gboolean server_signal_notified = FALSE;
+static int server_signal_pipe[2];
 
 /* Wipe cookie files */
 void
@@ -141,8 +143,16 @@ gdm_server_stop (GdmDisplay *disp)
 	    /* avoid SIGCHLD race */
 	    disp->servpid = 0;
 
-	    if (kill (servpid, SIGTERM) == 0)
+	    gdm_debug ("gdm_server_stop: Killing server pid %d", (int)servpid);
+
+	    gdm_sigchld_block_push ();
+	    if (servpid > 0 &&
+		kill (servpid, SIGTERM) == 0)
 		    waitpid (servpid, 0, 0);
+	    gdm_sigchld_block_pop ();
+
+
+	    gdm_debug ("gdm_server_stop: Server pid %d dead", (int)servpid);
     }
 
     gdm_server_wipe_cookies (disp);
@@ -154,17 +164,17 @@ busy_ask_user (GdmDisplay *disp)
     /* if we have "open" we can talk to the user */
     if (access (EXPANDED_SBINDIR "/gdmopen", X_OK) == 0) {
 	    char *error = g_strdup_printf
-		    (_("There already appears to be an X server "
-		       "running on display %s.  Should I try another "
-		       "display number?  If you answer no, I will "
-		       "attempt to start the server on %s again.%s"),
+		    (gdm_cons_i18n (N_("There already appears to be an X server "
+				       "running on display %s.  Should I try another "
+				       "display number?  If you answer no, I will "
+				       "attempt to start the server on %s again.%s")),
 		     disp->name,
 		     disp->name,
 #ifdef __linux__
-		     _("  (You can change consoles by pressing Ctrl-Alt "
-		       "plus a function key, such as Ctrl-Alt-F7 to go "
-		       "to console 7.  X servers usually run on consoles "
-		       "7 and higher.)")
+		     gdm_cons_i18n (N_("  (You can change consoles by pressing Ctrl-Alt "
+				       "plus a function key, such as Ctrl-Alt-F7 to go "
+				       "to console 7.  X servers usually run on consoles "
+				       "7 and higher.)"))
 #else /* ! __linux__ */
 		     /* no info for non linux users */
 		     ""
@@ -230,8 +240,8 @@ display_busy (GdmDisplay *disp)
 		 * of course additions are welcome to make this more robust */
 		if (strstr (buf, "Server is already active for display")
 		    == buf) {
-			gdm_error (_("Display %s is busy, there is another "
-				     "X server already running"),
+			gdm_error (_("Display %s is busy. There is another "
+				     "X server running already."),
 				   disp->name);
 			fclose (fp);
 			return TRUE;
@@ -321,10 +331,17 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     gdm_debug ("gdm_server_start: %s", d->name);
 
     /* Create new cookie */
-    if ( ! gdm_auth_secure_display (d))
+    if ( ! gdm_auth_secure_display (d)) 
 	    return FALSE;
     gdm_slave_send_string (GDM_SOP_COOKIE, d->cookie);
-    ve_setenv ("DISPLAY", d->name, TRUE);
+    gnome_setenv ("DISPLAY", d->name, TRUE);
+
+    if (pipe (server_signal_pipe) != 0) {
+	    gdm_error (_("%s: Error opening a pipe: %s"),
+		       "gdm_server_start", g_strerror (errno));
+	    return FALSE; 
+    }
+    server_signal_notified = FALSE;
 
     /* Catch USR1 from X server */
     usr1.sa_handler = gdm_server_usr1_handler;
@@ -332,7 +349,10 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     sigemptyset (&usr1.sa_mask);
 
     if (sigaction (SIGUSR1, &usr1, &old_usr1) < 0) {
-	    gdm_error (_("gdm_server_start: Error setting up USR1 signal handler"));
+	    gdm_error (_("%s: Error setting up USR1 signal handler: %s"),
+		       "gdm_server_start", g_strerror (errno));
+	    close (server_signal_pipe[0]);
+	    close (server_signal_pipe[1]);
 	    return FALSE;
     }
 
@@ -342,8 +362,11 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     sigemptyset (&chld.sa_mask);
 
     if (sigaction (SIGCHLD, &chld, &old_chld) < 0) {
-	    gdm_error (_("gdm_server_start: Error setting up CHLD signal handler"));
+	    gdm_error (_("%s: Error setting up CHLD signal handler: %s"),
+		       "gdm_server_start", g_strerror (errno));
 	    sigaction (SIGUSR1, &old_usr1, NULL);
+	    close (server_signal_pipe[0]);
+	    close (server_signal_pipe[1]);
 	    return FALSE;
     }
 
@@ -353,9 +376,12 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     sigemptyset (&alrm.sa_mask);
 
     if (sigaction (SIGALRM, &alrm, &old_alrm) < 0) {
-	    gdm_error (_("gdm_server_start: Error setting up ALRM signal handler"));
+	    gdm_error (_("%s: Error setting up ALRM signal handler: %s"),
+		       "gdm_server_start", g_strerror (errno));
 	    sigaction (SIGUSR1, &old_usr1, NULL);
 	    sigaction (SIGCHLD, &old_chld, NULL);
+	    close (server_signal_pipe[0]);
+	    close (server_signal_pipe[1]);
 	    return FALSE;
     }
 
@@ -390,7 +416,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 		     * use USR1 of course, so try openning the display 
 		     * as a test, but the */
 
-		    ve_setenv ("XAUTHORITY", d->authfile, TRUE);
+		    gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
 		    for (i = 0;
 			 d->dsp == NULL &&
 			 d->servstat == SERVER_STARTED &&
@@ -402,14 +428,25 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 			    else
 				    d->servstat = SERVER_RUNNING;
 		    }
-		    ve_unsetenv ("XAUTHORITY");
+		    gnome_unsetenv ("XAUTHORITY");
 		    if (d->dsp == NULL &&
 			/* Note: we could have still gotten a SIGCHLD */
 			d->servstat == SERVER_STARTED) {
 			    d->servstat = SERVER_TIMEOUT;
 		    }
 	    } else {
-		    gdm_run ();
+		    fd_set rfds;
+
+		    gdm_debug ("gdm_server_start: Before mainloop waiting for server");
+
+		    FD_ZERO (&rfds);
+		    FD_SET (server_signal_pipe[0], &rfds);
+
+		    do {
+			    select (server_signal_pipe[0]+1, &rfds, NULL, NULL, NULL);
+		    } while ( ! server_signal_notified);
+
+		    gdm_debug ("gdm_server_start: After mainloop waiting for server");
 	    }
     }
 
@@ -438,6 +475,9 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 	    sigaction (SIGCHLD, &old_chld, NULL);
 	    sigaction (SIGALRM, &old_alrm, NULL);
 
+	    close (server_signal_pipe[0]);
+	    close (server_signal_pipe[1]);
+
 	    if (SERVER_IS_FLEXI (d))
 		    gdm_slave_send_num (GDM_SOP_FLEXI_OK, 0 /* bogus */);
 #ifdef __linux__
@@ -456,9 +496,13 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     /* bad things are happening */
     if (d->servpid > 0) {
-	    if (kill (d->servpid, SIGTERM) == 0)
-		    waitpid (d->servpid, NULL, 0);
+	    pid_t pid = d->servpid;
 	    d->servpid = 0;
+	    gdm_sigchld_block_push ();
+	    if (pid > 0 &&
+		kill (pid, SIGTERM) == 0)
+		    waitpid (pid, NULL, 0);
+	    gdm_sigchld_block_pop ();
     }
 
     /* We will rebake cookies anyway, so wipe these */
@@ -471,6 +515,9 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     sigaction (SIGCHLD, &old_chld, NULL);
     sigaction (SIGALRM, &old_alrm, NULL);
 
+    close (server_signal_pipe[0]);
+    close (server_signal_pipe[1]);
+
     if (disp->type == TYPE_FLEXI_XNEST &&
 	display_xnest_no_connect (disp)) {
 	    gdm_slave_send_num (GDM_SOP_FLEXI_ERR,
@@ -530,16 +577,40 @@ void
 gdm_server_checklog (GdmDisplay *disp)
 {
 #ifdef __linux__
-	    if (d->vt < 0 &&
-		(d->type == TYPE_LOCAL ||
-		 d->type == TYPE_FLEXI)) {
-		    d->vt = display_vt (d);
-		    if (d->vt >= 0)
-			    gdm_slave_send_num (GDM_SOP_VT_NUM, d->vt);
-	    }
+	if (d->vt < 0 &&
+	    (d->type == TYPE_LOCAL ||
+	     d->type == TYPE_FLEXI)) {
+		d->vt = display_vt (d);
+		if (d->vt >= 0)
+			gdm_slave_send_num (GDM_SOP_VT_NUM, d->vt);
+	}
 #endif
 }
 
+static void
+rotate_logs (const char *dname)
+{
+	/* I'm too lazy to write a loop damnit */
+	char *fname4 = g_strconcat (GdmLogDir, "/", dname, ".log.4", NULL);
+	char *fname3 = g_strconcat (GdmLogDir, "/", dname, ".log.3", NULL);
+	char *fname2 = g_strconcat (GdmLogDir, "/", dname, ".log.2", NULL);
+	char *fname1 = g_strconcat (GdmLogDir, "/", dname, ".log.1", NULL);
+	char *fname = g_strconcat (GdmLogDir, "/", dname, ".log", NULL);
+
+	/* Rotate the logs (keep 4 last) */
+	unlink (fname4);
+	rename (fname3, fname4);
+	rename (fname2, fname3);
+	rename (fname1, fname2);
+	rename (fname, fname1);
+
+	g_free (fname4);
+	g_free (fname3);
+	g_free (fname2);
+	g_free (fname1);
+	g_free (fname);
+}
+
 /**
  * gdm_server_spawn:
  * @disp: Pointer to a GdmDisplay structure
@@ -558,6 +629,7 @@ gdm_server_spawn (GdmDisplay *d)
     int len, i;
     const char *command;
     char *bin;
+    pid_t pid;
 
     if (d == NULL ||
 	ve_string_empty (d->command)) {
@@ -566,25 +638,32 @@ gdm_server_spawn (GdmDisplay *d)
 
     d->servstat = SERVER_STARTED;
 
+    gdm_sigchld_block_push ();
+
     /* eek, some previous copy, just wipe it */
     if (d->servpid > 0) {
-	    if (kill (d->servpid, SIGTERM) == 0)
-		    waitpid (d->servpid, NULL, 0);
+	    pid_t pid = d->servpid;
+	    d->servpid = 0;
+	    if (kill (pid, SIGTERM) == 0)
+		    waitpid (pid, NULL, 0);
     }
-    
+
     /* Fork into two processes. Parent remains the gdm process. Child
      * becomes the X server. */
 
-    gdm_safe_fork (&(d->servpid));
+    gdm_sigterm_block_push ();
+    pid = d->servpid = fork ();
+    gdm_sigterm_block_pop ();
+    gdm_sigchld_block_pop ();
     
-    switch (d->servpid) {
+    switch (pid) {
 	
     case 0:
         alarm (0);
 
 	/* Close the XDMCP fd inherited by the daemon process */
 	if (GdmXdmcp)
-	    gdm_xdmcp_close();
+		gdm_xdmcp_close();
 
 	/* close things */
 	for (i = 0; i < sysconf (_SC_OPEN_MAX); i++)
@@ -596,9 +675,12 @@ gdm_server_spawn (GdmDisplay *d)
 	open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
 	open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
 
+	/* Rotate the X server logs */
+	rotate_logs (d->name);
+
         /* Log all output from spawned programs to a file */
 	logfd = open (g_strconcat (GdmLogDir, "/", d->name, ".log", NULL),
-		      O_CREAT|O_TRUNC|O_WRONLY, 0666);
+		      O_CREAT|O_TRUNC|O_WRONLY, 0644);
 
 	if (logfd != -1) {
 		dup2 (logfd, 1);
@@ -632,29 +714,25 @@ gdm_server_spawn (GdmDisplay *d)
 	sigemptyset (&dfl_signal.sa_mask);
 
 	if (sigaction (SIGHUP, &dfl_signal, NULL) < 0) {
-	    gdm_error (_("gdm_server_spawn: Error setting HUP to SIG_DFL"));
-	    _exit (SERVER_ABORT);
+		gdm_error (_("gdm_server_spawn: Error setting HUP to SIG_DFL"));
+		_exit (SERVER_ABORT);
 	}
 	if (sigaction (SIGTERM, &dfl_signal, NULL) < 0) {
-	    gdm_error (_("gdm_server_spawn: Error setting TERM to SIG_DFL"));
-	    _exit (SERVER_ABORT);
+		gdm_error (_("gdm_server_spawn: Error setting TERM to SIG_DFL"));
+		_exit (SERVER_ABORT);
 	}
 
-	/* unblock HUP/TERM/USR1 so that we can control the
-	 * X server */
-	sigprocmask (SIG_SETMASK, &sysmask, NULL);
+	/* unblock signals (especially HUP/TERM/USR1) so that we
+	 * can control the X server */
 	sigemptyset (&mask);
-	sigaddset (&mask, SIGUSR1);
-	sigaddset (&mask, SIGHUP);
-	sigaddset (&mask, SIGTERM);
-	sigprocmask (SIG_UNBLOCK, &mask, NULL);
+	sigprocmask (SIG_SETMASK, &mask, NULL);
 
 	if (d->type == TYPE_FLEXI_XNEST) {
-		ve_setenv ("DISPLAY", d->xnest_disp, TRUE);
+		gnome_setenv ("DISPLAY", d->xnest_disp, TRUE);
 		if (d->xnest_auth_file != NULL)
-			ve_setenv ("XAUTHORITY", d->xnest_auth_file, TRUE);
+			gnome_setenv ("XAUTHORITY", d->xnest_auth_file, TRUE);
 		else
-			ve_unsetenv ("XAUTHORITY");
+			gnome_unsetenv ("XAUTHORITY");
 	}
 
 	bin = ve_first_word (d->command);
@@ -730,12 +808,12 @@ gdm_server_spawn (GdmDisplay *d)
 			_exit (SERVER_ABORT);
 		}
 		if (pwent->pw_dir != NULL &&
-		    g_file_exists (pwent->pw_dir))
-			ve_setenv ("HOME", pwent->pw_dir, TRUE);
+		    g_file_test (pwent->pw_dir, G_FILE_TEST_EXISTS))
+			gnome_setenv ("HOME", pwent->pw_dir, TRUE);
 		else
-			ve_setenv ("HOME", "/", TRUE); /* Hack */
-		ve_setenv ("SHELL", pwent->pw_shell, TRUE);
-		ve_unsetenv ("MAIL");
+			gnome_setenv ("HOME", "/", TRUE); /* Hack */
+		gnome_setenv ("SHELL", pwent->pw_shell, TRUE);
+		gnome_unsetenv ("MAIL");
 
 		if (setgid (pwent->pw_gid) < 0)  {
 			gdm_error (_("%s: Couldn't set groupid to %d"), 
@@ -780,12 +858,11 @@ gdm_server_spawn (GdmDisplay *d)
 	break;
 	
     default:
-	gdm_debug ("gdm_server_spawn: Forked server on pid %d", (int)d->servpid);
+	gdm_debug ("gdm_server_spawn: Forked server on pid %d", (int)pid);
 	break;
     }
 }
 
-
 /**
  * gdm_server_usr1_handler:
  * @sig: Signal value
@@ -797,10 +874,13 @@ static void
 gdm_server_usr1_handler (gint sig)
 {
     d->servstat = SERVER_RUNNING; /* Server ready to accept connections */
+    d->starttime = time (NULL);
 
     gdm_debug ("gdm_server_usr1_handler: Got SIGUSR1, server running");
 
-    gdm_quit ();
+    server_signal_notified = TRUE;
+    /* this will quit the select */
+    write (server_signal_pipe[1], "Yay!", 4);
 }
 
 
@@ -818,7 +898,9 @@ gdm_server_alarm_handler (gint signal)
 
     gdm_debug ("gdm_server_alarm_handler: Got SIGALRM, server abort");
 
-    gdm_quit ();
+    server_signal_notified = TRUE;
+    /* this will quit the select */
+    write (server_signal_pipe[1], "Yay!", 4);
 }
 
 
@@ -854,10 +936,13 @@ gdm_server_child_handler (int signal)
 			d->servstat = SERVER_ABORT;	/* Server died unexpectedly */
 			d->servpid = 0;
 
-			gdm_quit ();
+			server_signal_notified = TRUE;
+			/* this will quit the select */
+			write (server_signal_pipe[1], "Yay!", 4);
 		} else if (pid == extra_process) {
 			/* an extra process died, yay! */
 			extra_process = -1;
+			extra_status = status;
 		}
 	}
 }
@@ -888,7 +973,7 @@ gdm_server_alloc (gint id, const gchar *command)
     d->userauth = NULL;
     d->command = g_strdup (command);
     d->cookie = NULL;
-    d->dispstat = DISPLAY_DEAD;
+    d->dispstat = DISPLAY_UNBORN;
     d->greetpid = 0;
     d->name = g_strdup_printf (":%d", id);  
     d->hostname = g_strdup (hostname);
@@ -919,6 +1004,9 @@ gdm_server_alloc (gint id, const gchar *command)
     d->login = NULL;
 
     d->timed_login_ok = FALSE;
+
+    d->slave_notify_conn = NULL;
+    d->master_notify_fd = -1;
     
     return d;
 }
