@@ -21,7 +21,8 @@
  * selected hostname will be printed on stdout. */
 
 #include <config.h>
-#include <gnome.h>
+#include <libgnome/libgnome.h>
+#include <libgnomeui/libgnomeui.h>
 #include <glade/glade.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -46,13 +47,10 @@
 
 #include <viciousui.h>
 
-#include "gdmchooser.h"
 #include "gdm.h"
 #include "misc.h"
 #include "gdmwm.h"
 
-static const gchar RCSid[]="$Id: gdmchooser.c,v 1.29 2001/08/25 21:46:15 jirka Exp $";
-
 /* set the DOING_GDM_DEVELOPMENT env variable if you want to
  * search for the glade file in the current dir and not the system
  * install dir, better then something you have to change
@@ -61,26 +59,39 @@ static gboolean DOING_GDM_DEVELOPMENT = FALSE;
 
 static gboolean RUNNING_UNDER_GDM = FALSE;
 
+typedef struct _GdmChooserHost GdmChooserHost;
+struct _GdmChooserHost {
+    gchar *name;
+    gchar *desc;
+    struct in_addr ia;
+    GdkPixbuf *picture;
+    gboolean willing;
+};
+
+
 static const gchar *scanning_message = N_("Please wait: scanning local network for XDMCP-enabled hosts...");
 static const gchar *empty_network = N_("No serving hosts were found.");
 static const gchar *active_network = N_("Choose a host to connect to from the selection below.");
 static gchar *glade_filename = NULL;
 
-static GdmChooserHost * gdm_chooser_host_alloc (const char *hostname,
-						const char *description,
-						struct in_addr *ia,
-						gboolean willing);
-
-static gboolean gdm_chooser_decode_packet (GIOChannel   *source,
-					   GIOCondition  condition,
-					   gpointer      data);
+/* XDM chooser style stuff */
+static gchar *xdm_address = NULL;
+static gchar *client_address = NULL;
+static gint connection_type = 0;
 
 static void gdm_chooser_abort (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
-static void gdm_chooser_browser_update (void);
-static void gdm_chooser_xdmcp_init (char **hosts);
-static void gdm_chooser_host_dispose (GdmChooserHost *host);
-static void gdm_chooser_choose_host (const gchar *hostname);
-static void gdm_chooser_add_hosts (char **hosts);
+
+/* Exported for glade */
+gboolean gdm_chooser_cancel (void);
+gboolean gdm_chooser_manage (GtkButton *button, gpointer data);
+gboolean gdm_chooser_browser_select (GtkWidget *widget,
+				     gint selected,
+				     GdkEvent *event);
+gboolean gdm_chooser_browser_unselect (GtkWidget *widget,
+				       gint selected,
+				       GdkEvent *event);
+gboolean gdm_chooser_xdmcp_discover (void);
+void display_chooser_information (void);
 
 static guint scan_time_handler = 0;
 
@@ -134,10 +145,27 @@ static GtkWidget *status_label;
 
 static GIOChannel *channel;
 static GList *hosts = NULL;
-static GdkImlibImage *defhostimg;
+static GdkPixbuf *defhostimg;
 static GnomeIconList *browser;
 static GdmChooserHost *curhost;
 
+static void
+gdm_chooser_host_dispose (GdmChooserHost *host)
+{
+    if (!host)
+	return;
+
+    if (host->picture != NULL)
+	    gdk_pixbuf_unref (host->picture);
+    host->picture = NULL;
+
+    g_free (host->name);
+    host->name = NULL;
+    g_free (host->desc);
+    host->desc = NULL;
+    g_free (host);
+}
+
 static gint 
 gdm_chooser_sort_func (gpointer d1, gpointer d2)
 {
@@ -150,6 +178,117 @@ gdm_chooser_sort_func (gpointer d1, gpointer d2)
     return strcmp (a->name, b->name);
 }
 
+static GdmChooserHost * 
+gdm_chooser_host_alloc (const char *hostname,
+			const char *description,
+			struct in_addr *ia,
+			gboolean willing)
+{
+    GdmChooserHost *host;
+    GdkPixbuf *img;
+    gchar *hostimg;
+    GList *hostl;
+
+    host = g_new0 (GdmChooserHost, 1);
+    host->name = g_strdup (hostname);
+    host->desc = g_strdup (description);
+    memcpy (&host->ia, ia, sizeof (struct in_addr));
+    host->willing = willing;
+
+    hostl = g_list_find_custom (hosts,
+				host,
+				(GCompareFunc) gdm_chooser_sort_func);
+    /* replace */
+    if (hostl != NULL) {
+	    GdmChooserHost *old = hostl->data;
+	    hostl->data = host;
+	    gdm_chooser_host_dispose (old);
+    } else {
+	    hosts = g_list_insert_sorted (hosts, 
+					  host,
+					  (GCompareFunc) gdm_chooser_sort_func);
+    }
+    
+    if ( ! willing)
+	    return host;
+
+    hostimg = g_strconcat (GdmHostIconDir, "/", hostname, NULL);
+
+    if (access (hostimg, R_OK) == 0 &&
+	(img = gdk_pixbuf_new_from_file (hostimg, NULL)) != NULL) {
+	gint w, h;
+
+	w = gdk_pixbuf_get_width (img);
+	h = gdk_pixbuf_get_height (img);
+	
+	if (w>h && w>GdmIconMaxWidth) {
+	    h = h * ((gfloat) GdmIconMaxWidth/w);
+	    w = GdmIconMaxWidth;
+	} else if (h>GdmIconMaxHeight) {
+	    w = w * ((gfloat) GdmIconMaxHeight/h);
+	    h = GdmIconMaxHeight;
+	}
+
+
+	if (w != gdk_pixbuf_get_width (img) ||
+	    h != gdk_pixbuf_get_height (img))
+		host->picture = gdk_pixbuf_scale_simple (img, w, h,
+							 GDK_INTERP_BILINEAR);
+	else
+		host->picture = gdk_pixbuf_ref (img);
+	
+	gdk_pixbuf_unref (img);
+    } else if (defhostimg != NULL) {
+	    host->picture = gdk_pixbuf_ref (defhostimg);
+    }
+
+    g_free (hostimg);
+
+    return host;
+}
+
+static void
+gdm_chooser_browser_update (void)
+{
+    GList *li;
+    gboolean any;
+
+    gnome_icon_list_freeze (GNOME_ICON_LIST (browser));
+    gnome_icon_list_clear (GNOME_ICON_LIST (browser));
+
+    any = FALSE;
+    for (li = hosts; li != NULL; li = li->next) {
+	    GdmChooserHost *host = (GdmChooserHost *) li->data;
+
+	    if (host->willing) {
+		    /* FIXME: the \n doesn't actually propagate
+		     * since the icon list is a broken piece of horsedung */
+		    char *temp = g_strconcat (host->name, " \n",
+					      host->desc, NULL);
+		    gnome_icon_list_append_pixbuf (GNOME_ICON_LIST (browser),
+						   host->picture,
+						   NULL /* icon_filename */,
+						   temp);
+		    g_free (temp);
+		    any = TRUE;
+	    }
+    }
+
+    gnome_icon_list_thaw (GNOME_ICON_LIST (browser));
+
+    if (any) {
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (chooser_app, "status_label")),
+			  _(active_network));
+    } else {
+      gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (chooser_app, "status_label")),
+			  _(empty_network));
+    }
+    gtk_widget_set_sensitive (GTK_WIDGET (manage), FALSE);
+    gtk_widget_set_sensitive (GTK_WIDGET (rescan), TRUE);
+    gtk_widget_set_sensitive (GTK_WIDGET (browser), TRUE);
+}
+
+
 static gboolean
 gdm_host_known (struct in_addr *ia)
 {
@@ -264,51 +403,60 @@ gdm_chooser_decode_packet (GIOChannel   *source,
 static void
 gdm_chooser_find_bcaddr (void)
 {
-    gint i = 0;
-    struct ifconf ifcfg;
-    struct ifreq ifr[MAXIF];
-    struct in_addr *ia;
+	int i = 0, num;
+	struct ifconf ifc;
+	char *buf;
+	struct ifreq *ifr;
+	struct in_addr *ia;
 
-    ifcfg.ifc_buf = (gchar *) &ifr;
-    ifcfg.ifc_len = sizeof (ifr);
-
-    memset (ifr, 0, sizeof (ifr));
-
-    if (ioctl (sockfd, SIOCGIFCONF, &ifcfg) < 0) 
-	gdm_chooser_abort ("Could not get SIOCIFCONF");
-
-    for (i=0 ; i < MAXIF ; i++) 
-	if (strlen (ifr[i].ifr_name)) {
-	    struct ifreq *ifreq;
-	    struct sockaddr_in *ba;
-
-	    ifreq = g_new0 (struct ifreq, 1);
-
-	    strncpy (ifreq->ifr_name, ifr[i].ifr_name, sizeof (ifr[i].ifr_name));
-
-	    if (ioctl (sockfd, SIOCGIFFLAGS, ifreq) < 0) 
-		gdm_chooser_abort ("Could not get SIOCGIFFLAGS for %s", ifr[i].ifr_name);
-
-	    if ((ifreq->ifr_flags & IFF_UP) == 0)
-		goto done;
-
-	    if ((ifreq->ifr_flags & IFF_BROADCAST) == 0)
-		goto done;
-
-	    if (ioctl (sockfd, SIOCGIFBRDADDR, ifreq) < 0)
-		goto done;
-
-	    ba = (struct sockaddr_in *) &ifreq->ifr_broadaddr;
-
-	    ia = g_new0 (struct in_addr, 1);
-
-	    ia->s_addr = ba->sin_addr.s_addr;
-
-	    bcaddr = g_slist_append (bcaddr, ia);
-
-	done:
-	    g_free (ifreq);
+#ifdef SIOCGIFNUM
+	if (ioctl (sockfd, SIOCGIFNUM, &num) < 0) {
+		num = 64;
 	}
+#else
+	num = 64;
+#endif
+
+	ifc.ifc_len = sizeof (struct ifreq) * num;
+	ifc.ifc_buf = buf = g_malloc (ifc.ifc_len);
+	if (ioctl (sockfd, SIOCGIFCONF, &ifc) < 0) {
+		g_free (buf);
+		gdm_chooser_abort ("Cannot get local addresses!");
+		return;
+	}
+
+	ifr = ifc.ifc_req;
+	for (i = 0 ; i < num ; i++) {
+		if ( ! ve_string_empty (ifr[i].ifr_name)) {
+			struct ifreq ifreq;
+			struct sockaddr_in *ba = NULL;
+
+			memset (&ifreq, 0, sizeof (ifreq));
+
+			strncpy (ifreq.ifr_name, ifr[i].ifr_name,
+				 sizeof (ifreq.ifr_name));
+			/* paranoia */
+			ifreq.ifr_name[sizeof (ifreq.ifr_name) - 1] = '\0';
+
+			if (ioctl (sockfd, SIOCGIFFLAGS, &ifreq) < 0) 
+				gdm_chooser_abort ("Could not get SIOCGIFFLAGS for %s", ifr[i].ifr_name);
+
+			if ((ifreq.ifr_flags & IFF_UP) == 0 ||
+			    (ifreq.ifr_flags & IFF_BROADCAST) == 0 ||
+			    ioctl (sockfd, SIOCGIFBRDADDR, &ifreq) < 0)
+				continue;
+
+			ba = (struct sockaddr_in *) &ifreq.ifr_broadaddr;
+
+			ia = g_new0 (struct in_addr, 1);
+
+			ia->s_addr = ba->sin_addr.s_addr;
+
+			bcaddr = g_slist_append (bcaddr, ia);
+		}
+	}
+
+	g_free (buf);
 }
 
 static gboolean
@@ -368,8 +516,8 @@ gdm_chooser_xdmcp_discover (void)
     gtk_widget_set_sensitive (GTK_WIDGET (manage), FALSE);
     gnome_icon_list_clear (GNOME_ICON_LIST (browser));
     gtk_widget_set_sensitive (GTK_WIDGET (browser), FALSE);
-    gtk_label_set (GTK_LABEL (status_label),
-		   _(scanning_message));
+    gtk_label_set_text (GTK_LABEL (status_label),
+			_(scanning_message));
 
     while (hl) {
 	gdm_chooser_host_dispose ((GdmChooserHost *) hl->data);
@@ -396,6 +544,69 @@ gdm_chooser_xdmcp_discover (void)
     return TRUE;
 }
 
+#ifndef ishexdigit
+#define ishexdigit(c) (isdigit(c) || ('a' <= (c) && (c) <= 'f'))
+#endif
+#define HexChar(c)  ('0' <= (c) && (c) <= '9' ? (c) - '0' : (c) - 'a' + 10)
+
+static int
+from_hex (const char *s, char *d, int len)
+{
+  int t;
+  while (len >= 2)
+    {
+      if (!ishexdigit(*s))
+ return 1;
+      t = HexChar (*s) << 4;
+      s++;
+      if (!ishexdigit(*s))
+ return 1;
+      t += HexChar (*s);
+      s++;
+      *d++ = t;
+      len -= 2;
+    }
+  return len;
+}
+
+static void
+gdm_chooser_add_hosts (char **hosts)
+{
+	struct hostent *hostent;
+	struct sockaddr_in qa;
+	struct in_addr *ia;
+	int i;
+
+	for (i = 0; hosts != NULL && hosts[i] != NULL; i++) {
+		const char *name = hosts[i];
+
+		if (strcmp (name, "BROADCAST") == 0) {
+			gdm_chooser_find_bcaddr ();
+			continue;
+		}
+		if (strlen (name) == 8 &&
+		    from_hex (name, (char *) &qa.sin_addr, strlen (name)) == 0)
+			qa.sin_family = AF_INET;
+		else if ((qa.sin_addr.s_addr = inet_addr (name)) != -1)
+			qa.sin_family = AF_INET;
+		else if ((hostent = gethostbyname (name)) != NULL
+			 && hostent->h_addrtype == AF_INET
+			 && hostent->h_length == 4) {
+			qa.sin_family = AF_INET;
+			memmove (&qa.sin_addr, hostent->h_addr, 4);
+		} else {
+			continue; /* not a valid address */
+		}
+
+		ia = g_new0 (struct in_addr, 1);
+		ia->s_addr = qa.sin_addr.s_addr;
+		queryaddr = g_slist_append (queryaddr, ia);
+	}
+
+	if (bcaddr == NULL &&
+	    queryaddr == NULL)
+		gdm_chooser_find_bcaddr ();
+}
 
 static void
 gdm_chooser_xdmcp_init (char **hosts)
@@ -436,6 +647,60 @@ gdm_chooser_xdmcp_init (char **hosts)
     gdm_chooser_xdmcp_discover();
 }
 
+static void
+gdm_chooser_choose_host (const char *hostname)
+{
+  ARRAY8 tmparr;
+  struct hostent *hentry;
+
+  g_print ("%s\n", curhost->name);
+  if (xdm_address != NULL) {
+      struct sockaddr_in in_addr;
+      char xdm_addr[32];
+      char client_addr[32];
+      int fd;
+      char buf[1024];
+      XdmcpBuffer buffer;
+      long family, port, addr;
+      if (strlen (xdm_address) > 64 ||
+	  from_hex (xdm_address, xdm_addr, strlen (xdm_address)) != 0)
+	      gdm_chooser_abort ("gdm_chooser_chooser_host: Invalid xdm address.");
+
+      family = (xdm_addr[0] << 8) | xdm_addr[1];
+      port = (xdm_addr[2] << 8) | xdm_addr[3];
+      addr = (xdm_addr[4] << 24) | (xdm_addr[5] << 16) | (xdm_addr[6] << 8) | xdm_addr[7];
+      in_addr.sin_family = AF_INET;
+      in_addr.sin_port = htons (port);
+      in_addr.sin_addr.s_addr = htonl (addr);
+      if ((fd = socket (PF_INET, SOCK_STREAM, 0)) == -1)
+	      gdm_chooser_abort ("gdm_chooser_chooser_host: Couldn't create response socket.");
+
+      if (connect (fd, (struct sockaddr *) &in_addr, sizeof (in_addr)) == -1)
+	      gdm_chooser_abort ("gdm_chooser_chooser_host: Couldn't connect to xdm.");
+      buffer.data = (BYTE *) buf;
+      buffer.size = sizeof (buf);
+      buffer.pointer = 0;
+      buffer.count = 0;
+
+      if (strlen (client_address) > 64 || from_hex (client_address, client_addr, strlen (client_address)) != 0)
+	      gdm_chooser_abort ("gdm_chooser_chooser_host: Invalid client address.");
+      tmparr.data = (BYTE *) client_addr;
+      tmparr.length = strlen (client_address) / 2;
+      XdmcpWriteARRAY8 (&buffer, &tmparr);
+      XdmcpWriteCARD16 (&buffer, (CARD16) connection_type);
+      hentry = gethostbyname (hostname);
+      if (!hentry)
+	      gdm_chooser_abort ("gdm_chooser_chooser_host: Couldn't get host entry for %s", hostname);
+
+      tmparr.data = (BYTE *) hentry->h_addr_list[0]; /* XXX */
+      tmparr.length = 4;
+
+      XdmcpWriteARRAY8 (&buffer, &tmparr);
+      write (fd, (char *) buffer.data, buffer.pointer);
+      close (fd);
+  }
+}
+
 
 gboolean
 gdm_chooser_cancel (void)
@@ -593,58 +858,19 @@ gdm_chooser_browser_unselect (GtkWidget *widget, gint selected, GdkEvent *event)
     return TRUE;
 }
 
-
-static void
-gdm_chooser_browser_update (void)
-{
-    GList *li;
-    gboolean any;
-
-    gnome_icon_list_freeze (GNOME_ICON_LIST (browser));
-    gnome_icon_list_clear (GNOME_ICON_LIST (browser));
-
-    any = FALSE;
-    for (li = hosts; li != NULL; li = li->next) {
-	    GdmChooserHost *host = (GdmChooserHost *) li->data;
-
-	    if (host->willing) {
-		    /* FIXME: the \n doesn't actually propagate
-		     * since the icon list is a broken piece of horsedung */
-		    char *temp = g_strconcat (host->name, " \n",
-					      host->desc, NULL);
-		    gnome_icon_list_append_imlib (GNOME_ICON_LIST (browser),
-						  host->picture, temp);
-		    g_free (temp);
-		    any = TRUE;
-	    }
-    }
-
-    gnome_icon_list_thaw (GNOME_ICON_LIST (browser));
-
-    if (any) {
-      gtk_label_set (GTK_LABEL (glade_xml_get_widget (chooser_app, "status_label")),
-		     _(active_network));
-    } else {
-      gtk_label_set (GTK_LABEL (glade_xml_get_widget (chooser_app, "status_label")),
-		     _(empty_network));
-    }
-    gtk_widget_set_sensitive (GTK_WIDGET (manage), FALSE);
-    gtk_widget_set_sensitive (GTK_WIDGET (rescan), TRUE);
-    gtk_widget_set_sensitive (GTK_WIDGET (browser), TRUE);
-}
-
 void
 display_chooser_information (void)
 {
-   glade_xml_new (glade_filename, "gdmchooser_help_dialog");
+   glade_xml_new (glade_filename, "gdmchooser_help_dialog", PACKAGE);
 }
 
 static void 
 gdm_chooser_gui_init (void)
 {
+#if 0
     GtkStyle  *style;
     GdkColor  bbg = { 0, 0xFFFF, 0xFFFF, 0xFFFF };
- 
+#endif
    
         /* Look for the glade file in $(datadir)/gdmconfig or, failing that,
      * look in the current directory.
@@ -653,7 +879,7 @@ gdm_chooser_gui_init (void)
      */
 	
     if ( ! DOING_GDM_DEVELOPMENT) {
-	    if (g_file_exists (GDM_GLADE_DIR "/gdmchooser.glade")) {
+	    if (g_file_test (GDM_GLADE_DIR "/gdmchooser.glade", G_FILE_TEST_EXISTS)) {
 		    glade_filename = g_strdup (GDM_GLADE_DIR
 					       "/gdmchooser.glade");
 	    } else {
@@ -673,19 +899,23 @@ gdm_chooser_gui_init (void)
     if (access (GdmHostDefaultIcon, R_OK) != 0)
 	gdm_chooser_abort (_("Can't open default host icon: %s"), GdmHostDefaultIcon);
     else
-	defhostimg = gdk_imlib_load_image (GdmHostDefaultIcon);
+	defhostimg = gdk_pixbuf_new_from_file (GdmHostDefaultIcon, NULL);
 
     /* Main window */
     g_assert (glade_filename != NULL);
-    chooser_app = glade_xml_new (glade_filename, "gdmchooser_main");
+    chooser_app = glade_xml_new (glade_filename, "gdmchooser_main", PACKAGE);
     if (chooser_app == NULL) {
 	    GtkWidget *fatal_error = 
-		    gnome_error_dialog(_("Cannot find the glade interface description\n"
-					 "file, cannot run gdmchooser.\n"
-					 "Please check your installation and the\n"
-					 "location of the gdmchooser.glade file."));
-	    gnome_dialog_run_and_close(GNOME_DIALOG(fatal_error));
-	    exit(EXIT_FAILURE);
+		    gtk_message_dialog_new (NULL /* parent */,
+					    GTK_DIALOG_MODAL /* flags */,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("Cannot find the glade interface description\n"
+					      "file, cannot run gdmchooser.\n"
+					      "Please check your installation and the\n"
+					      "location of the gdmchooser.glade2 file."));
+	    gtk_dialog_run (GTK_DIALOG (fatal_error));
+	    exit (EXIT_FAILURE);
     }
     glade_xml_signal_autoconnect (chooser_app);
    
@@ -701,13 +931,18 @@ gdm_chooser_gui_init (void)
 	cancel == NULL ||
 	status_label == NULL) {
 	    GtkWidget *fatal_error = 
-		    gnome_error_dialog(_("The glade interface description file\n"
-					 "appears to be corrupted.\n"
-					 "Please check your installation."));
-	    gnome_dialog_run_and_close(GNOME_DIALOG(fatal_error));
+		    gtk_message_dialog_new (NULL /* parent */,
+					    GTK_DIALOG_MODAL /* flags */,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    _("The glade interface description file\n"
+					      "appears to be corrupted.\n"
+					      "Please check your installation."));
+	    gtk_dialog_run (GTK_DIALOG (fatal_error));
 	    exit(EXIT_FAILURE);
 	}
    
+#if 0
     /* Find background style for browser */
     style = gtk_style_copy (chooser->style);
    
@@ -716,6 +951,7 @@ gdm_chooser_gui_init (void)
      */
     style->bg[GTK_STATE_NORMAL] = bbg;
     gtk_widget_push_style (style);
+#endif
 
     browser = (GnomeIconList *) glade_xml_get_widget (chooser_app, 
 						     "chooser_iconlist");
@@ -723,12 +959,14 @@ gdm_chooser_gui_init (void)
     gnome_icon_list_set_separators (GNOME_ICON_LIST (browser), " /-_.");
     gnome_icon_list_set_icon_width (GNOME_ICON_LIST (browser), GdmIconMaxWidth + 20);
     gnome_icon_list_set_icon_border (GNOME_ICON_LIST (browser), 2);
+#if 0
     gtk_widget_pop_style();
+#endif
     gnome_icon_list_thaw (GNOME_ICON_LIST (browser));
 
-    gtk_widget_set_usize (GTK_WIDGET (chooser), 
-			  (gint) gdk_screen_width() * 0.4, 
-			  (gint) gdk_screen_height() * 0.6);
+    gtk_widget_set_size_request (GTK_WIDGET (chooser), 
+				 (gint) gdk_screen_width() * 0.4, 
+				 (gint) gdk_screen_height() * 0.6);
 
     gdm_wm_center_window (GTK_WINDOW (chooser));
 }
@@ -762,219 +1000,20 @@ gdm_chooser_signals_init (void)
 	gdm_chooser_abort (_("Could not set signal mask!"));
 }
 
-static GdmChooserHost * 
-gdm_chooser_host_alloc (const char *hostname,
-			const char *description,
-			struct in_addr *ia,
-			gboolean willing)
-{
-    GdmChooserHost *host;
-    GdkImlibImage *imlibimg;
-    gchar *hostimg;
-    GList *hostl;
-
-    host = g_malloc (sizeof (GdmChooserHost));
-    host->name = g_strdup (hostname);
-    host->desc = g_strdup (description);
-    memcpy (&host->ia, ia, sizeof (struct in_addr));
-    host->willing = willing;
-
-    hostl = g_list_find_custom (hosts,
-				host,
-				(GCompareFunc) gdm_chooser_sort_func);
-    /* replace */
-    if (hostl != NULL) {
-	    GdmChooserHost *old = hostl->data;
-	    hostl->data = host;
-	    gdm_chooser_host_dispose (old);
-    } else {
-	    hosts = g_list_insert_sorted (hosts, 
-					  host,
-					  (GCompareFunc) gdm_chooser_sort_func);
-    }
-    
-    if ( ! willing)
-	    return host;
-
-    hostimg = g_strconcat (GdmHostIconDir, "/", hostname, NULL);
-
-    if (access (hostimg, R_OK) == 0 &&
-	(imlibimg = gdk_imlib_load_image (hostimg))) {
-	gint w, h;
-
-	w = imlibimg->rgb_width;
-	h = imlibimg->rgb_height;
-	
-	if (w>h && w>GdmIconMaxWidth) {
-	    h = h * ((gfloat) GdmIconMaxWidth/w);
-	    w = GdmIconMaxWidth;
-	} else if (h>GdmIconMaxHeight) {
-	    w = w * ((gfloat) GdmIconMaxHeight/h);
-	    h = GdmIconMaxHeight;
-	}
-
-	host->picture = gdk_imlib_clone_scaled_image (imlibimg, w, h);
-	
-	gdk_imlib_destroy_image (imlibimg);
-    } else {
-	host->picture = defhostimg;
-    }
-
-    g_free (hostimg);
-
-    return host;
-}
-
-
-static void
-gdm_chooser_host_dispose (GdmChooserHost *host)
-{
-    if (!host)
-	return;
-
-    if (host->picture != defhostimg)
-	gdk_imlib_destroy_image (host->picture);
-    host->picture = NULL;
-
-    g_free (host->name);
-    host->name = NULL;
-    g_free (host->desc);
-    host->desc = NULL;
-    g_free (host);
-}
-
-static gchar *xdm_address = NULL;
-static gchar *client_address = NULL;
-static gint connection_type = 0;
-
 struct poptOption xdm_options [] = {
-  { "xdmaddress", '\0', POPT_ARG_STRING|POPT_ARGFLAG_ONEDASH, &xdm_address, 0,
-    N_("Socket for xdm communication"), N_("SOCKET") }, { "clientaddress", '\0', POPT_ARG_STRING|POPT_ARGFLAG_ONEDASH, &client_address, 0, N_("Client address to return in response to xdm"), N_("ADDRESS") },
-  { "connectionType", '\0', POPT_ARG_INT|POPT_ARGFLAG_ONEDASH, &connection_type, 0, N_("Connection type to return in response to xdm"), N_("TYPE") },
-  POPT_AUTOHELP
-  { NULL, 0, 0, NULL, 0}
+	{ "xdmaddress", '\0', POPT_ARG_STRING | POPT_ARGFLAG_ONEDASH,
+	  &xdm_address, 0,
+	  N_("Socket for xdm communication"), N_("SOCKET") },
+        { "clientaddress", '\0', POPT_ARG_STRING | POPT_ARGFLAG_ONEDASH,
+	  &client_address, 0,
+	  N_("Client address to return in response to xdm"), N_("ADDRESS") },
+        { "connectionType", '\0', POPT_ARG_INT|POPT_ARGFLAG_ONEDASH,
+	  &connection_type, 0,
+	  N_("Connection type to return in response to xdm"), N_("TYPE") },
+        POPT_AUTOHELP
+	{ NULL, 0, 0, NULL, 0}
 };
 
-#ifndef ishexdigit
-#define ishexdigit(c) (isdigit(c) || ('a' <= (c) && (c) <= 'f'))
-#endif
-#define HexChar(c)  ('0' <= (c) && (c) <= '9' ? (c) - '0' : (c) - 'a' + 10)
-
-static int
-from_hex (const char *s, char *d, int len)
-{
-  int t;
-  while (len >= 2)
-    {
-      if (!ishexdigit(*s))
- return 1;
-      t = HexChar (*s) << 4;
-      s++;
-      if (!ishexdigit(*s))
- return 1;
-      t += HexChar (*s);
-      s++;
-      *d++ = t;
-      len -= 2;
-    }
-  return len;
-}
-
-
-static void
-gdm_chooser_choose_host (const char *hostname)
-{
-  ARRAY8 tmparr;
-  struct hostent *hentry;
-
-  g_print ("%s\n", curhost->name);
-  if (xdm_address)
-    {
-      struct sockaddr_in in_addr;
-      char xdm_addr[32];
-      char client_addr[32];
-      int fd;
-      char buf[1024];
-      XdmcpBuffer buffer;
-      long family, port, addr;
-      if (strlen (xdm_address) > 64 ||
-	  from_hex (xdm_address, xdm_addr, strlen (xdm_address)) != 0)
-	      gdm_chooser_abort ("gdm_chooser_chooser_host: Invalid xdm address.");
-
-      family = (xdm_addr[0] << 8) | xdm_addr[1];
-      port = (xdm_addr[2] << 8) | xdm_addr[3];
-      addr = (xdm_addr[4] << 24) | (xdm_addr[5] << 16) | (xdm_addr[6] << 8) | xdm_addr[7];
-      in_addr.sin_family = AF_INET;
-      in_addr.sin_port = htons (port);
-      in_addr.sin_addr.s_addr = htonl (addr);
-      if ((fd = socket (PF_INET, SOCK_STREAM, 0)) == -1)
-	      gdm_chooser_abort ("gdm_chooser_chooser_host: Couldn't create response socket.");
-
-      if (connect (fd, (struct sockaddr *) &in_addr, sizeof (in_addr)) == -1)
-	      gdm_chooser_abort ("gdm_chooser_chooser_host: Couldn't connect to xdm.");
-      buffer.data = (BYTE *) buf;
-      buffer.size = sizeof (buf);
-      buffer.pointer = 0;
-      buffer.count = 0;
-
-      if (strlen (client_address) > 64 || from_hex (client_address, client_addr, strlen (client_address)) != 0)
-	      gdm_chooser_abort ("gdm_chooser_chooser_host: Invalid client address.");
-      tmparr.data = (BYTE *) client_addr;
-      tmparr.length = strlen (client_address) / 2;
-      XdmcpWriteARRAY8 (&buffer, &tmparr);
-      XdmcpWriteCARD16 (&buffer, (CARD16) connection_type);
-      hentry = gethostbyname (hostname);
-      if (!hentry)
-	      gdm_chooser_abort ("gdm_chooser_chooser_host: Couldn't get host entry for %s", hostname);
-
-      tmparr.data = (BYTE *) hentry->h_addr_list[0]; /* XXX */
-      tmparr.length = 4;
-
-      XdmcpWriteARRAY8 (&buffer, &tmparr);
-      write (fd, (char *) buffer.data, buffer.pointer);
-      close (fd);
-    }
-}
-
-static void
-gdm_chooser_add_hosts (char **hosts)
-{
-	struct hostent *hostent;
-	struct sockaddr_in qa;
-	struct in_addr *ia;
-	int i;
-
-	for (i = 0; hosts != NULL && hosts[i] != NULL; i++) {
-		const char *name = hosts[i];
-
-		if (strcmp (name, "BROADCAST") == 0) {
-			gdm_chooser_find_bcaddr ();
-			continue;
-		}
-		if (strlen (name) == 8 &&
-		    from_hex (name, (char *) &qa.sin_addr, strlen (name)) == 0)
-			qa.sin_family = AF_INET;
-		else if ((qa.sin_addr.s_addr = inet_addr (name)) != -1)
-			qa.sin_family = AF_INET;
-		else if ((hostent = gethostbyname (name)) != NULL
-			 && hostent->h_addrtype == AF_INET
-			 && hostent->h_length == 4) {
-			qa.sin_family = AF_INET;
-			memmove (&qa.sin_addr, hostent->h_addr, 4);
-		} else {
-			continue; /* not a valid address */
-		}
-
-		ia = g_new0 (struct in_addr, 1);
-		ia->s_addr = qa.sin_addr.s_addr;
-		queryaddr = g_slist_append (queryaddr, ia);
-	}
-
-	if (bcaddr == NULL &&
-	    queryaddr == NULL)
-		gdm_chooser_find_bcaddr ();
-}
-
 static void
 set_background (void)
 {
@@ -1002,6 +1041,27 @@ set_background (void)
 	}
 }
 
+static gboolean
+gdm_event (GSignalInvocationHint *ihint,
+	   guint		n_param_values,
+	   const GValue	       *param_values,
+	   gpointer		data)
+{
+	/* HAAAAAAAAAAAAAAAAACK */
+	/* Since the user has not logged in yet and may have left/right
+	 * mouse buttons switched, we just translate every right mouse click
+	 * to a left mouse click */
+	GdkEvent *event = g_value_get_pointer ((GValue *)param_values);
+	if ((event->type == GDK_BUTTON_PRESS ||
+	     event->type == GDK_2BUTTON_PRESS ||
+	     event->type == GDK_3BUTTON_PRESS ||
+	     event->type == GDK_BUTTON_RELEASE)
+	    && event->button.button == 3)
+		event->button.button = 1;
+
+	return TRUE;
+}      
+
 int 
 main (int argc, char *argv[])
 {
@@ -1010,15 +1070,13 @@ main (int argc, char *argv[])
     char **hosts;
     poptContext ctx;
     const char *gdm_version;
+    GnomeProgram *program;
 
     if (g_getenv ("DOING_GDM_DEVELOPMENT") != NULL)
 	    DOING_GDM_DEVELOPMENT = TRUE;
     if (g_getenv ("RUNNING_UNDER_GDM") != NULL)
 	    RUNNING_UNDER_GDM = TRUE;
 
-    /* Avoid creating ~gdm/.gnome stuff */
-    gnome_do_not_create_directories = TRUE;
-
     openlog ("gdmchooser", LOG_PID, LOG_DAEMON);
 
     fixedargc = argc + 1;
@@ -1028,12 +1086,27 @@ main (int argc, char *argv[])
 	fixedargv[i] = argv[i];
     
     fixedargv[fixedargc-1] = "--disable-sound";
-    gnome_init_with_popt_table ("gdmchooser", VERSION, fixedargc, fixedargv, xdm_options, 0, &ctx);
+
+    bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+    textdomain (GETTEXT_PACKAGE);
+
+    program = gnome_program_init ("gdmchooser", VERSION, 
+				  /* FIXME: oh fuck this inits way too much
+				   * shit that we don't want */
+				  LIBGNOMEUI_MODULE /* module_info */,
+				  fixedargc, fixedargv,
+				  GNOME_PARAM_POPT_TABLE, xdm_options,
+				  GNOME_PARAM_CREATE_DIRECTORIES, FALSE,
+				  NULL);
+
+    g_object_get (G_OBJECT (program),
+		  GNOME_PARAM_POPT_CONTEXT, &ctx,
+		  NULL);
     glade_gnome_init();
     g_free (fixedargv);
 
-    bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-    textdomain (PACKAGE);
+    gdm_chooser_parse_config();
 
     gdm_wm_screen_init (GdmXineramaScreen);
 
@@ -1041,30 +1114,30 @@ main (int argc, char *argv[])
 
     if (gdm_version != NULL &&
 	strcmp (gdm_version, VERSION) != 0) {
-	    char *msg;
 	    GtkWidget *dialog;
 
 	    gdm_wm_init (0);
 
 	    gdm_wm_focus_new_windows (TRUE);
 
-	    msg = g_strdup_printf
-		    (_("The chooser version (%s) does not match the daemon "
-		       "version (%s).\n"
-		       "You have probably just upgraded gdm.\n"
-		       "Please restart the gdm daemon or reboot the computer."),
-		     VERSION, gdm_version);
-	    dialog = gnome_error_dialog (msg);
-	    g_free (msg);
+	    dialog = gtk_message_dialog_new (NULL /* parent */,
+					     GTK_DIALOG_MODAL /* flags */,
+					     GTK_MESSAGE_ERROR,
+					     GTK_BUTTONS_OK,
+					     _("The chooser version (%s) does not match the daemon "
+					       "version (%s).\n"
+					       "You have probably just upgraded gdm.\n"
+					       "Please restart the gdm daemon or reboot the computer."),
+					     VERSION, gdm_version);
 
 	    gtk_widget_show_all (dialog);
 	    gdm_wm_center_window (GTK_WINDOW (dialog));
-	    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+
+	    gtk_dialog_run (GTK_DIALOG (dialog));
 
 	    return EXIT_SUCCESS;
     }
     
-    gdm_chooser_parse_config();
     gdm_chooser_gui_init();
     gdm_chooser_signals_init();
 
@@ -1083,6 +1156,16 @@ main (int argc, char *argv[])
     gdm_chooser_xdmcp_init (hosts);
     poptFreeContext (ctx);
 
+    if (g_getenv ("RUNNING_UNDER_GDM") != NULL) {
+	    guint sid = g_signal_lookup ("event",
+					 GTK_TYPE_WIDGET);
+	    g_signal_add_emission_hook (sid,
+					0 /* detail */,
+					gdm_event,
+					NULL /* data */,
+					NULL /* destroy_notify */);
+    }
+
     gtk_widget_show_now (chooser);
 
     /* can it ever happen that it'd be NULL here ??? */
