@@ -19,7 +19,8 @@
  */
 
 #include "config.h"
-#include <gnome.h>
+#include <libgnome/libgnome.h>
+#include <libgnomeui/libgnomeui.h>
 #include <gdk/gdkx.h>
 #include <X11/X.h>
 #include <X11/Xlib.h>
@@ -32,8 +33,6 @@
 #include "gdmwm.h"
 #include "gdm.h"
 
-static const gchar RCSid[]="$Id: gdmwm.c,v 1.7 2001/09/06 22:52:14 jirka Exp $";
-
 typedef struct _GdmWindow GdmWindow;
 struct _GdmWindow {
 	int x, y;
@@ -43,6 +42,7 @@ struct _GdmWindow {
 	gboolean ignore_size_hints; /* for gdm windows */
 	gboolean center; /* do centering */
 	gboolean recenter; /* do re-centering */
+        gboolean takefocus; /* permit take focus */
 };
 
 static GList *windows = NULL;
@@ -52,10 +52,10 @@ static Display *wm_disp = NULL;
 static Window wm_root = None;
 static Window wm_login_window = None;
 
-static gulong XA_WM_PROTOCOLS = 0;
-static gulong XA_WM_STATE = 0;
-static gulong XA_WM_TAKE_FOCUS = 0;
-static gulong XA_COMPOUND_TEXT = 0;
+static Atom XA_WM_PROTOCOLS = 0;
+static Atom XA_WM_STATE = 0;
+static Atom XA_WM_TAKE_FOCUS = 0;
+static Atom XA_COMPOUND_TEXT = 0;
 
 static int trap_depth = 0;
 
@@ -171,7 +171,7 @@ gdm_wm_center_window (GtkWindow *cw)
 	if (y < gdm_wm_screen.y)
 		y = gdm_wm_screen.y;
 
- 	gtk_widget_set_uposition (GTK_WIDGET (cw), x, y);	
+ 	gtk_window_move (GTK_WINDOW (cw), x, y);	
 }
 
 
@@ -280,8 +280,9 @@ get_typed_property_data (Display *xdisplay,
 	  if (format_returned == 8 && type_returned == XA_COMPOUND_TEXT)
 	    {
 	      gchar **tlist = NULL;
-	      gint count = gdk_text_property_to_text_list (type_returned, 8, prop_data,
-							   nitems_return, &tlist);
+	      gint count = gdk_text_property_to_text_list
+		      (gdk_x11_xatom_to_atom (type_returned), 8, prop_data,
+		       nitems_return, &tlist);
 
 	      if (count && tlist && tlist[0])
 		{
@@ -357,15 +358,50 @@ wm_protocol_check_support (Window xwin,
   return is_supported;
 }
 
+static GList *
+find_window_list (Window w, gboolean deco_ok)
+{
+	GList *li;
+
+	for (li = windows; li != NULL; li = li->next) {
+		GdmWindow *gw = li->data;
+
+		if (gw->win == w)
+			return li;
+		if (deco_ok &&
+		    (gw->deco == w ||
+		     gw->shadow == w))
+			return li;
+	}
+
+	return NULL;
+}
+
+static GdmWindow *
+find_window (Window w, gboolean deco_ok)
+{
+	GList *li = find_window_list (w, deco_ok);
+	if (li == NULL)
+		return NULL;
+	else
+		return li->data;
+}
+
 void
 gdm_wm_focus_window (Window window)
 {
 	XWindowAttributes attribs = {0};
+	GdmWindow *win;
 
 	if (no_focus_login > 0 &&
 	    window == wm_login_window)
 		return;
 
+	win = find_window (window, TRUE);
+	if (win != NULL &&
+	    ! win->takefocus)
+		return;
+
 	trap_push ();
 
 	XGetWindowAttributes (wm_disp, window, &attribs);
@@ -395,35 +431,6 @@ gdm_wm_focus_window (Window window)
 	trap_pop ();
 }
 
-static GList *
-find_window_list (Window w, gboolean deco_ok)
-{
-	GList *li;
-
-	for (li = windows; li != NULL; li = li->next) {
-		GdmWindow *gw = li->data;
-
-		if (gw->win == w)
-			return li;
-		if (deco_ok &&
-		    (gw->deco == w ||
-		     gw->shadow == w))
-			return li;
-	}
-
-	return NULL;
-}
-
-static GdmWindow *
-find_window (Window w, gboolean deco_ok)
-{
-	GList *li = find_window_list (w, deco_ok);
-	if (li == NULL)
-		return NULL;
-	else
-		return li->data;
-}
-
 static void
 center_x_window (GdmWindow *gw, Window w, Window hintwin)
 {
@@ -597,8 +604,6 @@ add_deco (GdmWindow *w)
 				       0, 
 				       black, black);
 
-	XMapWindow (wm_disp, w->deco);
-
 	XGetWindowAttributes (wm_disp, w->deco, &attribs);
 	XSelectInput (wm_disp, w->deco,
 		      attribs.your_event_mask |
@@ -606,6 +611,8 @@ add_deco (GdmWindow *w)
 		      SubstructureNotifyMask |
 		      SubstructureRedirectMask);
 
+	XMapWindow (wm_disp, w->deco);
+
 	XReparentWindow (wm_disp, w->win, w->deco, 1, 1);
 
 	trap_pop ();
@@ -635,6 +642,7 @@ add_window (Window w, gboolean center)
 	gw = find_window (w, FALSE);
 	if (gw == NULL) {
 		XClassHint hint = { NULL, NULL };
+		XWMHints *wmhints;
 		int x, y;
 		Window root;
 		unsigned int width, height, border, depth;
@@ -649,6 +657,17 @@ add_window (Window w, gboolean center)
 		gw->ignore_size_hints = FALSE;
 		gw->center = center;
 		gw->recenter = FALSE;
+		gw->takefocus = TRUE;
+
+		wmhints = XGetWMHints (wm_disp, w);
+		if (wmhints != NULL) {
+			/* NoInput windows */
+			if ((wmhints->flags & InputHint) &&
+			    ! wmhints->input) {
+				gw->takefocus = FALSE;
+			}
+			XFree (wmhints);
+		}
 
 		/* hack, set USpos/size on login window */
 		if (w == wm_login_window) {
@@ -670,6 +689,9 @@ add_window (Window w, gboolean center)
 				 * such stuff */
 				gw->center = FALSE;
 				gw->recenter = FALSE;
+			} else if (is_wm_class (&hint, "xscribble", 0)) {
+				/* hack, xscribble mustn't take focus */
+				gw->takefocus = FALSE;
 			}
 			if (hint.res_name != NULL)
 				XFree (hint.res_name);
@@ -830,7 +852,7 @@ event_process (XEvent *ev)
 		if (gw == NULL) {
 			if (ev->xmaprequest.parent == wm_root) {
 				XGrabServer (wm_disp);
-				add_window (w, TRUE /*center*/);
+				gw = add_window (w, TRUE /*center*/);
 				XUngrabServer (wm_disp);
 			}
 		}
@@ -960,19 +982,15 @@ event_process (XEvent *ev)
 static GPollFD event_poll_fd;
 
 static gboolean  
-event_prepare (gpointer  source_data, 
-	       GTimeVal *current_time,
-	       gint     *timeout,
-	       gpointer  user_data)
+event_prepare (GSource *source,
+	       gint     *timeout)
 {
 	*timeout = -1;
 	return XPending (wm_disp);
 }
 
 static gboolean  
-event_check (gpointer  source_data,
-	     GTimeVal *current_time,
-	     gpointer  user_data)
+event_check (GSource *source)
 {
 	if (event_poll_fd.revents & G_IO_IN) {
 		return XPending (wm_disp);
@@ -992,9 +1010,9 @@ process_events (void)
 }
 
 static gboolean  
-event_dispatch (gpointer  source_data,
-		GTimeVal *current_time,
-		gpointer  user_data)
+event_dispatch (GSource     *source,
+		GSourceFunc  callback,
+		gpointer     user_data)
 {
 	process_events ();
 
@@ -1004,13 +1022,17 @@ event_dispatch (gpointer  source_data,
 static GSourceFuncs event_funcs = {
 	event_prepare,
 	event_check,
-	event_dispatch,
-	(GDestroyNotify)g_free
+	event_dispatch
 };
 
+#if 0
+/* ALEX: WTF is this? It's not used by anything, and I don't want
+ * the greeter to depend on libgnomeui and libbonoboui.
+ */
+
 /* HAAAAAAAAAAAAAACK */
 void
-gnome_dock_item_grab_pointer (GnomeDockItem *item)
+bonobo_dock_item_grab_pointer (BonoboDockItem *item)
 {
   GdkCursor *fleur;
 
@@ -1029,15 +1051,17 @@ gnome_dock_item_grab_pointer (GnomeDockItem *item)
 
   gdk_cursor_destroy (fleur);
 }
+#endif
 
 void
 gdm_wm_init (Window login_window)
 {
 	XWindowAttributes attribs = { 0, };
+	GSource *source;
 
 	wm_login_window = login_window;
 
-	wm_disp = XOpenDisplay (gdk_display_name);
+	wm_disp = XOpenDisplay (gdk_get_display ());
 	if (wm_disp == NULL) {
 		/* EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEK! */
 		wm_disp = GDK_DISPLAY ();
@@ -1050,20 +1074,22 @@ gdm_wm_init (Window login_window)
 	XA_WM_STATE = XInternAtom (wm_disp, "WM_STATE", False);
 	XA_WM_TAKE_FOCUS = XInternAtom (wm_disp, "WM_TAKE_FOCUS", False);
 
-	XA_COMPOUND_TEXT = gdk_atom_intern ("COMPOUND_TEXT", FALSE);
+	XA_COMPOUND_TEXT = XInternAtom (wm_disp, "COMPOUND_TEXT", False);
 
 
 	wm_root = DefaultRootWindow (wm_disp);
 
 	add_all_current_windows ();
 
-	g_source_add (GDK_PRIORITY_EVENTS, FALSE, &event_funcs, NULL, NULL, NULL);
+	source = g_source_new (&event_funcs, sizeof (GSource));
 
 	event_poll_fd.fd = ConnectionNumber (wm_disp);
 	event_poll_fd.events = G_IO_IN;
 
-	g_main_add_poll (&event_poll_fd, GDK_PRIORITY_EVENTS);
-
+	g_source_add_poll (source, &event_poll_fd);
+	g_source_set_priority (source, GDK_PRIORITY_EVENTS);
+	g_source_set_can_recurse (source, FALSE);
+	g_source_attach (source, NULL);
 
 	/* set event mask for events on root window */
 	XGetWindowAttributes (wm_disp, wm_root, &attribs);
